package com.care.common.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.care.common.entity.CareEventOrderChamb;
import com.care.common.entity.CareHouse;
import com.care.common.enums.ChambOrderStatusEnum;
import com.care.common.mapper.CareEventOrderChambMapper;
import com.care.common.service.CareEventOrderChambService;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.order.ChambEventOrderCountVO;
import com.care.common.vo.order.ChambEventOrderVO;
import org.springframework.stereotype.Service;

/**
 * 管家工单(CareEventOrderChamb)表服务实现类
 *
 * @author makejava
 * @since 2021-06-02 23:31:34
 */
@Service
public class CareEventOrderChambServiceImpl extends ServiceImpl<CareEventOrderChambMapper, CareEventOrderChamb> implements CareEventOrderChambService {

    /**
     * 待处理的工单
     *
     * @param pageReqVo
     * @param chambId
     * @return
     */
    @Override
    public IPage<ChambEventOrderVO> queryTodoEventByChambId(PageReqVO pageReqVo, Long chambId) {
        IPage<ChambEventOrderVO> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        IPage<ChambEventOrderVO> pageRes = this.baseMapper.queryTodoEventByChambId(page,chambId);
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            pageRes.getRecords().forEach(item -> {
                item.setStatusName(ChambOrderStatusEnum.getCodeToName(item.getStatus()));
            });
        }
        return pageRes;
    }

    /**
     * 进行中的工单
     *
     * @param pageReqVo
     * @param chambId
     * @return
     */
    @Override
    public IPage<ChambEventOrderVO> queryDoingEventByChambId(PageReqVO pageReqVo, Long chambId) {
        IPage<ChambEventOrderVO> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        IPage<ChambEventOrderVO> pageRes = this.baseMapper.queryDoingEventByChambId(page,chambId);
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            pageRes.getRecords().forEach(item -> {
                item.setStatusName(ChambOrderStatusEnum.getCodeToName(item.getStatus()));
            });
        }
        return pageRes;
    }

    /**
     * 完成的工单
     *
     * @param pageReqVo
     * @param chambId
     * @return
     */
    @Override
    public IPage<ChambEventOrderVO> queryDoneEventByChambId(PageReqVO pageReqVo, Long chambId) {
        IPage<ChambEventOrderVO> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        IPage<ChambEventOrderVO> pageRes = this.baseMapper.queryDoneEventByChambId(page,chambId);
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            pageRes.getRecords().forEach(item -> {
                item.setStatusName(ChambOrderStatusEnum.getCodeToName(item.getStatus()));
            });
        }
        return pageRes;
    }

    /**
     * 管家工单统计
     * @param chambId
     * @return
     */
    @Override
    public ChambEventOrderCountVO getOrderCountByChambId(Long chambId){
        return  this.baseMapper.getOrderCountByChambId(chambId);
    }

    @Override
    public CareEventOrderChamb getChambOrderByOrderId(Long orderId) {
        QueryWrapper<CareEventOrderChamb> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrderChamb::getOrderId,orderId);
        CareEventOrderChamb order = this.getOne(queryWrapper);
        return order;
    }

    @Override
    public ChambEventOrderVO getChambOrderByOrderIdAndChambId(Long orderId, Long chambId) {
        QueryWrapper<CareEventOrderChamb> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrderChamb::getOrderId,orderId).eq(CareEventOrderChamb::getChambId, chambId);
        CareEventOrderChamb order = this.getOne(queryWrapper);
        ChambEventOrderVO vo = new ChambEventOrderVO();
        BeanUtil.copyProperties(order, vo);
        return vo;
    }

}