package com.care.installation.service;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.care.common.cache.RedisUtil;
import com.care.common.constant.Constants;
import com.care.common.entity.*;
import com.care.common.enums.DeviceActiveStatusEnum;
import com.care.common.enums.DeviceStatusEnum;
import com.care.common.enums.InstallDevStatusEnum;
import com.care.common.service.*;
import com.care.common.util.JsonUtil;
import com.care.installation.vo.*;
import com.care.mqtt.service.MqttConnectorInstallPoolService;
import com.care.mqtt.service.MqttConnectorPoolService;
import com.care.mqtt.service.MqttPublishService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Slf4j
@Service
public class RoomService {


    @Resource
    private CareRoomService careRoomService;
    @Resource
    private CareRoomSensorService careRoomSensorService;
    @Resource
    private CareRoomGateService careRoomGateService;
    @Resource
    private CareRoomRegionService careRoomRegionService;
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private MqttPublishService mqttPublishService;
    @Resource
    private MqttConnectorInstallPoolService mqttConnectorInstallPoolService;
    @Resource
    private MqttConnectorPoolService mqttConnectorPoolService;
    @Resource
    private RedisUtil redisUtil;
    @Value("${mqtt.dev.jihuo.timeout:#{null}}")
    private String mqttJihuoTimeout;

    private static Logger logger = LoggerFactory.getLogger(RoomService.class);

    public  Map<String,Boolean> getConfStatusInfo(String devCode) {
        Map<String,Boolean> map = new HashMap<>();
        RoomVO roomVO = getRoomInfo(devCode);
        if (roomVO != null){
            map.put("roomVO",true);
        } else {
            map.put("roomVO",false);
        }
        RoomSensorVO roomSensorVO = getRoomSensorInfo(devCode);
        if (roomSensorVO != null){
            map.put("roomSensorVO",true);
        } else {
            map.put("roomSensorVO",false);
        }
        List<RoomGateVO> listRoomGateVO = listRoomGateInfo(devCode);
        if(CollUtil.isNotEmpty(listRoomGateVO)){
            map.put("listRoomGateVO",true);
        } else {
            map.put("listRoomGateVO",false);
        }

        List<RoomRegionVO> listRoomRegionVO = listRoomRegionInfo(devCode);
        if(CollUtil.isNotEmpty(listRoomRegionVO)){
            map.put("listRoomRegionVO",true);
        } else {
            map.put("listRoomRegionVO",false);
        }

        return map;
    }

    public RoomVO getRoomInfo(String devCode) {
        QueryWrapper<CareRoom> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoom::getRoomId,devCode);
        CareRoom careRoom = this.careRoomService.getOne(queryWrapper);
        if (careRoom != null){
            RoomVO roomVO = new RoomVO();
            BeanUtil.copyProperties(careRoom,roomVO);
            return roomVO;
        }
        return null;
    }

    public Boolean saveRoom(RoomVO vo) {
        if (StringUtils.isEmpty(vo.getRoomId())){
            return false;
        }

        CareRoom careRoom = new CareRoom();
        BeanUtil.copyProperties(vo,careRoom);

        QueryWrapper<CareRoom> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoom::getRoomId,careRoom.getRoomId());
        CareRoom careRoomDb = this.careRoomService.getOne(queryWrapper);
        if(careRoomDb != null ){
            careRoom.setId(careRoomDb.getId());
        }

        if (careRoom.getId() != null) {  //修改
            careRoom.setModifyTime(new Date());
            return this.careRoomService.updateById(careRoom);
        } else { //新增
            careRoom.setCreateTime(new Date());
            return this.careRoomService.save(careRoom);
        }
    }

    public RoomSensorVO getRoomSensorInfo(String devCode) {
        QueryWrapper<CareRoomSensor> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoomSensor::getRoomId,devCode);
        CareRoomSensor careRoomSensor = this.careRoomSensorService.getOne(queryWrapper);
        if (careRoomSensor != null){
            RoomSensorVO roomSensorVO = new RoomSensorVO();
            BeanUtil.copyProperties(careRoomSensor,roomSensorVO);
            return roomSensorVO;
        }
        return null;
    }

    public Boolean saveRoomSensor(RoomSensorVO vo) {
        if (StringUtils.isEmpty(vo.getRoomId())){
            return false;
        }

        CareRoomSensor careRoomSensor = new CareRoomSensor();
        BeanUtil.copyProperties(vo,careRoomSensor);

        QueryWrapper<CareRoomSensor> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoomSensor::getRoomId,careRoomSensor.getRoomId());
        CareRoomSensor careRoomSensorDb = this.careRoomSensorService.getOne(queryWrapper);
        if(careRoomSensorDb != null ){
            careRoomSensor.setId(careRoomSensorDb.getId());
        }

        if (careRoomSensor.getId() != null) {  //修改
            careRoomSensor.setModifyTime(new Date());
            return this.careRoomSensorService.updateById(careRoomSensor);
        } else { //新增
            careRoomSensor.setCreateTime(new Date());
            return this.careRoomSensorService.save(careRoomSensor);
        }
    }

    public List<RoomGateVO> listRoomGateInfo(String devCode) {
        QueryWrapper<CareRoomGate> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoomGate::getRoomId,devCode).orderByAsc(CareRoomGate::getGid);
        List<CareRoomGate> careRoomGateList = this.careRoomGateService.list(queryWrapper);
        if (CollUtil.isNotEmpty(careRoomGateList)){
            List<RoomGateVO> roomGateVOList = new ArrayList<>();
            careRoomGateList.forEach(item ->{
                RoomGateVO roomGateVO = new RoomGateVO();
                BeanUtil.copyProperties(item,roomGateVO);
                roomGateVOList.add(roomGateVO);
            });
            return roomGateVOList;
        }
        return null;
    }

    public RoomGateVO getRoomGateInfo(String devCode,Integer gid) {
        QueryWrapper<CareRoomGate> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoomGate::getRoomId,devCode).eq(CareRoomGate:: getGid,gid);
        CareRoomGate careRoomGate = this.careRoomGateService.getOne(queryWrapper);
        if (careRoomGate != null) {
            RoomGateVO roomGateVO = new RoomGateVO();
            BeanUtil.copyProperties(careRoomGate,roomGateVO);
            return roomGateVO;
        }
        return null;
    }

    public Boolean saveRoomGate(RoomGateVO vo) {
        if (StringUtils.isEmpty(vo.getRoomId())){
            return false;
        }
        CareRoomGate careRoomGate = new CareRoomGate();
        BeanUtil.copyProperties(vo,careRoomGate);
        if (careRoomGate.getId() != null) {  //修改
            careRoomGate.setModifyTime(new Date());
            return this.careRoomGateService.updateById(careRoomGate);
        } else { //新增
            careRoomGate.setCreateTime(new Date());
            return this.careRoomGateService.save(careRoomGate);
        }
    }

    public Boolean delRoomGate(String devCode,Integer gid) {
        QueryWrapper<CareRoomGate> removeWrapper = new QueryWrapper<>();
        removeWrapper.lambda().eq(CareRoomGate::getRoomId,devCode).eq(CareRoomGate::getGid,gid);
        return this.careRoomGateService.remove(removeWrapper);
    }


    public List<RoomRegionVO> listRoomRegionInfo(String devCode) {
        QueryWrapper<CareRoomRegion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoomRegion::getRoomId,devCode).orderByAsc(CareRoomRegion::getRid);
        List<CareRoomRegion> careRoomRegionList = this.careRoomRegionService.list(queryWrapper);
        if (CollUtil.isNotEmpty(careRoomRegionList)){
            List<RoomRegionVO> roomRegionVOList = new ArrayList<>();
            careRoomRegionList.forEach(item ->{
                RoomRegionVO roomRegionVO = new RoomRegionVO();
                BeanUtil.copyProperties(item,roomRegionVO);
                roomRegionVOList.add(roomRegionVO);
            });
            return roomRegionVOList;
        }
        return null;
    }

    public RoomRegionVO getRoomRegionInfo(String devCode,Integer rid) {
        QueryWrapper<CareRoomRegion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareRoomRegion::getRoomId,devCode).eq(CareRoomRegion::getRid,rid);
        CareRoomRegion careRoomRegion = this.careRoomRegionService.getOne(queryWrapper);
        if (careRoomRegion != null) {
            RoomRegionVO roomRegionVO = new RoomRegionVO();
            BeanUtil.copyProperties(careRoomRegion,roomRegionVO);
            return roomRegionVO;
        }
        return null;
    }

    public Boolean saveRoomRegion(RoomRegionVO vo) {
        if (StringUtils.isEmpty(vo.getRoomId())){
            return false;
        }
        CareRoomRegion careRoomRegion = new CareRoomRegion();
        BeanUtil.copyProperties(vo,careRoomRegion);
        if (careRoomRegion.getId() != null) {  //修改
            careRoomRegion.setModifyTime(new Date());
            return this.careRoomRegionService.updateById(careRoomRegion);
        } else { //新增
            careRoomRegion.setCreateTime(new Date());
            return this.careRoomRegionService.save(careRoomRegion);
        }
    }

    public Boolean delRoomRegion(String devCode,Integer rid) {
        QueryWrapper<CareRoomRegion> removeWrapper = new QueryWrapper<>();
        removeWrapper.lambda().eq(CareRoomRegion::getRoomId,devCode).eq(CareRoomRegion::getRid,rid);
        return this.careRoomRegionService.remove(removeWrapper);
    }
    /**
     * 发送Mqtt消息给dmp
     * @param devCode
     * @return
     */
    public Boolean commitRoomInfo(String devCode) {
        try {
            logger.info("commitRoomInfo.devCode == " + devCode);
            RoomVO roomVO = getRoomInfo(devCode);
            RoomSensorVO roomSensorVO = getRoomSensorInfo(devCode);
            List<RoomGateVO> roomGateVOList = listRoomGateInfo(devCode);
            List<RoomRegionVO> roomRegionVOList = listRoomRegionInfo(devCode);
            if (roomVO == null || roomSensorVO == null || CollUtil.isEmpty(roomGateVOList)) {
                return false;
            }

            QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(CareDevice::getDevCode, roomVO.getRoomId());
            CareDevice careDevice = careDeviceService.getOne(queryWrapper);
            if (careDevice == null) {
                return false;
            }

            //把act置为空
            UpdateWrapper<CareRoom> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(CareRoom::getRoomId,devCode)
            .set(CareRoom::getAck,null);
            careRoomService.update(updateWrapper);
            mqttPublishService.publish(careDevice, getRoomMqttData(roomVO),devCode,"careRoom");

            UpdateWrapper<CareRoomSensor> updateWrapper2 = new UpdateWrapper<>();
            updateWrapper2.lambda().eq(CareRoomSensor::getRoomId,devCode)
                    .set(CareRoomSensor::getAck,null);
            careRoomSensorService.update(updateWrapper2);
            mqttPublishService.publish(careDevice, getRoomSensorMqttData(roomSensorVO),devCode,"careRoomSensor");

            UpdateWrapper<CareRoomGate> updateWrapper3 = new UpdateWrapper<>();
            updateWrapper3.lambda().eq(CareRoomGate::getRoomId,devCode)
                    .set(CareRoomGate::getAck,null);
            careRoomGateService.update(updateWrapper3);
            mqttPublishService.publish(careDevice, getRoomGateVOMqttData(roomGateVOList), devCode,"careRoomGate");

            if (CollUtil.isNotEmpty(roomRegionVOList)){
                UpdateWrapper<CareRoomRegion> updateWrapper4 = new UpdateWrapper<>();
                updateWrapper4.lambda().eq(CareRoomRegion::getRoomId,devCode)
                        .set(CareRoomRegion::getAck,null);
                careRoomRegionService.update(updateWrapper4);
                mqttPublishService.publish(careDevice, getRoomRegionVOMqttData(roomRegionVOList),devCode,"careRoomRegion");
            }
            new Thread(() -> queryConfigureRoomInfoResult(devCode)).start();
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 启动一个线程，查询状态，防止前端不轮训，造成资源泄露
     */
    private void queryConfigureRoomInfoResult(String devCode) {
        Long timeout = Long.parseLong(mqttJihuoTimeout);
       try {
           Thread.sleep(timeout * 1000 + 100);
           getConfigureRoomInfoResult(devCode,false);
       } catch (Exception e){
           e.printStackTrace();
       }
    }
    /**
     * 查询配置房间信息的Mqtt返回结果
     * @return
     */
    public String getConfigureRoomInfoResult(String devCode,boolean isRepeat) {
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareDevice::getDevCode,devCode);
        CareDevice careDevice = careDeviceService.getOne(queryWrapper);

        QueryWrapper<CareRoom> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.lambda().eq(CareRoom::getRoomId,devCode);
        CareRoom careRoom = careRoomService.getOne(queryWrapper2);

        int careRoomResult = 0;
        if (careRoom == null || (StringUtils.isNotEmpty(careRoom.getAck()) && !"1".equals(careRoom.getAck()))) { //失败
            careRoomResult = 2;
        } else if ("1".equals(careRoom.getAck())) { //成功
            careRoomResult = 1;
        } else if (!redisUtil.hasKey(Constants.MQTT_DEV_JIHUO_TIMEOUT_KEY + ":" + devCode + ":" + "careRoom")) {
            careRoomResult = 3;
        } else {
            careRoomResult = 0;
        }

        QueryWrapper<CareRoomSensor> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.lambda().eq(CareRoomSensor::getRoomId,devCode);
        CareRoomSensor careRoomSensor = careRoomSensorService.getOne(queryWrapper3);

        int careRoomSensorResult = 0;

        if (careRoomSensor == null || (StringUtils.isNotEmpty(careRoomSensor.getAck()) && !"1".equals(careRoomSensor.getAck()))) { //失败
            careRoomSensorResult = 2;
        } else if ("1".equals(careRoomSensor.getAck())) { //成功
            careRoomSensorResult = 1;
        } else if (!redisUtil.hasKey(Constants.MQTT_DEV_JIHUO_TIMEOUT_KEY + ":" + devCode + ":" + "careRoomSensor")) {
            careRoomSensorResult = 3;
        } else {
            careRoomSensorResult = 0;
        }

        QueryWrapper<CareRoomGate> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.lambda().eq(CareRoomGate::getRoomId,devCode);
        List<CareRoomGate> careRoomGateList = careRoomGateService.list(queryWrapper4);
        int careRoomGateResult = 0;

         if(CollUtil.isNotEmpty(careRoomGateList)){
             CareRoomGate careRoomGate = careRoomGateList.get(0);
             if(careRoomGate == null || (StringUtils.isNotEmpty(careRoomGate.getAck()) && !"1".equals(careRoomGate.getAck()))) {
                careRoomGateResult = 2;
            } else if("1".equals(careRoomGate.getAck())) {
                careRoomGateResult = 1;
            } else if (!redisUtil.hasKey(Constants.MQTT_DEV_JIHUO_TIMEOUT_KEY + ":" + devCode + ":" + "careRoomGate")) {
                 careRoomGateResult = 3;
             } else {
                careRoomGateResult = 0;
            }
        } else { //不允许为空
            careRoomGateResult = 2;
        }

        QueryWrapper<CareRoomRegion> queryWrapper5 = new QueryWrapper<>();
        queryWrapper5.lambda().eq(CareRoomRegion::getRoomId,devCode);
        List<CareRoomRegion> careRoomRegionList = careRoomRegionService.list(queryWrapper5);
        int careRoomRegionResult = 0;

        if(CollUtil.isNotEmpty(careRoomRegionList)){
            CareRoomRegion careRoomRegion = careRoomRegionList.get(0);
           if (careRoomRegion == null|| (StringUtils.isNotEmpty(careRoomRegion.getAck()) && !"1".equals(careRoomRegion.getAck()))) {
                careRoomRegionResult = 2;
            }  else if("1".equals(careRoomRegion.getAck())) {
               careRoomRegionResult = 1;
            }  else if (!redisUtil.hasKey(Constants.MQTT_DEV_JIHUO_TIMEOUT_KEY + ":" + devCode + ":" + "careRoomRegion")) {
                careRoomRegionResult = 3;
            } else {
                careRoomRegionResult = 0;
            }
        } else {//允许为空
            careRoomRegionResult = 1;
        }

         if (careRoomResult == 2 || careRoomSensorResult == 2 || careRoomGateResult == 2 || careRoomRegionResult == 2) { //失败
             if(!isRepeat){
                 //设备离线
                 careDevice.setStatus(DeviceStatusEnum.OFFLINE.getValue());
                 mqttConnectorPoolService.removeOne(careDevice);
                 //未激活
                 careDevice.setActiveStatus(DeviceActiveStatusEnum.NOACTIVE.getValue());
                 careDeviceService.updateById(careDevice);

                 //去除publish的客户端监听
                 mqttConnectorInstallPoolService.removeInstallOne(careDevice);
             }

            return InstallDevStatusEnum.FAIL.getValue();
        }  else if (careRoomResult == 3 || careRoomSensorResult == 3 || careRoomGateResult == 3 || careRoomRegionResult == 3){ //超时
             if(!isRepeat) {
                 //去除publish的客户端监听
                 mqttConnectorInstallPoolService.removeInstallOne(careDevice);
             }
             return InstallDevStatusEnum.TIMEOUT.getValue();
         }  else if (careRoomResult == 0 || careRoomSensorResult == 0 || careRoomGateResult == 0 || careRoomRegionResult == 0){ //处理中
             return InstallDevStatusEnum.DOING.getValue();
        }  else { //成功
             if(!isRepeat) {
                 //激活
                 careDevice.setActiveStatus(DeviceActiveStatusEnum.ACTIVE.getValue());
                 //启动通道
                 mqttConnectorPoolService.addOne(careDevice);
                 //上线
                 careDevice.setStatus(DeviceStatusEnum.ONLINE.getValue());
                 careDevice.setActiveTime(DateUtil.date());
                 UpdateWrapper<CareDevice> updateWrapper = new UpdateWrapper<>();
                 updateWrapper.lambda().eq(CareDevice::getId,careDevice.getId())
                         .set(CareDevice::getStatus,careDevice.getStatus())
                         .set(CareDevice::getActiveStatus,careDevice.getActiveStatus())
                 .set(CareDevice::getActiveTime, careDevice.getActiveTime());
                 careDeviceService.update(updateWrapper);
                 //careDeviceService.updateById(careDevice);

                 //去除publish的客户端监听
                 mqttConnectorInstallPoolService.removeInstallOne(careDevice);
             }
            return InstallDevStatusEnum.SUCCESS.getValue();

        }
    }


    private String getRoomMqttData( RoomVO vo) throws Exception {
        Map<String,Object> data = new HashMap<>();
        data.put("type","RoomVO");
        Map<String,Object> msg = new HashMap<>();
        msg.put("roomId",vo.getRoomId());
        msg.put("roomX",vo.getX());
        msg.put("roomY",vo.getY());
        msg.put("roomZ",vo.getZ());
        data.put("msg",msg);
        return JsonUtil.toJson(data);
    }

    private String getRoomSensorMqttData( RoomSensorVO vo) throws Exception {
        Map<String,Object> data = new HashMap<>();
        data.put("type","RoomDeviceVO");
        Map<String,Object> msg = new HashMap<>();
        msg.put("roomDeviceId",vo.getRoomId());
        msg.put("roomId",vo.getRoomId());
        msg.put("deviceX",vo.getX());
        msg.put("deviceY",vo.getY());
        msg.put("deviceZ",vo.getZ());

        msg.put("deviceAtilt",0);
        msg.put("deviceEtilt",90);
        msg.put("deviceAtiltfov",60);
        msg.put("deviceEtiltfov",60);
        msg.put("deviceMount",0);
        data.put("msg",msg);
        return JsonUtil.toJson(data);
    }


    private String getRoomGateVOMqttData(List<RoomGateVO> list) throws Exception {
        Map<String,Object> data = new HashMap<>();
        data.put("type","RoomEntrancesVO");
        Map<String,Object> msg = new HashMap<>();
        List<Map<String,Object>> items = new ArrayList<>();
        list.forEach(roomGateVO ->{
            Map<String,Object> item = new HashMap<>();
            item.put("entranceId",roomGateVO.getRoomId()+"_"+roomGateVO.getGid());
            item.put("roomId",roomGateVO.getRoomId());
            item.put("entranceCode",roomGateVO.getGid());
            item.put("entranceType",1);
            item.put("direction",roomGateVO.getDirection());

            item.put("entranceLength",roomGateVO.getLength());
            item.put("width",roomGateVO.getWidth());
            item.put("hight",roomGateVO.getHeight());
            items.add(item);
        });
        msg.put("roomEntranceVOS",items);
        data.put("msg",msg);
        return JsonUtil.toJson(data);
    }

    private String getRoomRegionVOMqttData(List<RoomRegionVO> list) throws Exception {
        Map<String,Object> data = new HashMap<>();
        data.put("type","RoomRegionsVO");
        Map<String,Object> msg = new HashMap<>();
        List<Map<String,Object>> items = new ArrayList<>();
        list.forEach(roomRegionVO ->{
            Map<String,Object> item = new HashMap<>();
            item.put("regionId",roomRegionVO.getRoomId()+"_"+roomRegionVO.getRid());
            item.put("roomId",roomRegionVO.getRoomId());
            item.put("regionCode",roomRegionVO.getRid());
            item.put("regionType",roomRegionVO.getCls());
            item.put("positionX",roomRegionVO.getPositionX());
            item.put("positionY",roomRegionVO.getPositionY());
            item.put("scaleX",roomRegionVO.getScaleX());
            item.put("scaleY",roomRegionVO.getScaleY());
            item.put("scaleZ",roomRegionVO.getScaleZ());
            item.put("rotation",roomRegionVO.getRotation());
            items.add(item);
        });
        msg.put("roomRegionVOS",items);
        data.put("msg",msg);
        return JsonUtil.toJson(data);
    }
}
